@@ -1,61 +1,37 @@
 #!/usr/bin/env python3
 """
-K6GTE Contest logger
+not1mm Contest logger
 Email: michael.bridak@gmail.com
 GPL V3
+Class: BandMapWindow
+Purpose: Onscreen widget to show realtime spots from an AR cluster.
 """
 
 # pylint: disable=unused-import, c-extension-no-member, no-member, invalid-name, too-many-lines
-# pylint: disable=logging-fstring-interpolation, line-too-long
-
-from datetime import datetime, timezone
-from decimal import Decimal
-from json import loads
-from pathlib import Path
+# pylint: disable=logging-fstring-interpolation, line-too-long, no-name-in-module
 
 import logging
 import os
-
-# import pkgutil
 import platform
-import sys
 import sqlite3
+from datetime import datetime, timezone
+from decimal import Decimal
+from json import loads
 
-from PyQt5 import QtCore, QtGui
-from PyQt5 import QtNetwork
-from PyQt5 import QtWidgets, uic
+from PyQt6 import QtCore, QtGui, QtWidgets, uic, QtNetwork
+from PyQt6.QtGui import QColorConstants, QPalette, QColor
+from PyQt6.QtWidgets import QDockWidget
+from PyQt6.QtCore import pyqtSignal
 
-from not1mm.lib.multicast import Multicast
+import not1mm.fsutils as fsutils
 
-os.environ["QT_QPA_PLATFORMTHEME"] = "gnome"
+# from not1mm.lib.multicast import Multicast
+
+logger = logging.getLogger(__name__)
 
 PIXELSPERSTEP = 10
 UPDATE_INTERVAL = 2000
 
-# DeprecationWarning: 'pkgutil.get_loader' is deprecated and slated for removal in Python 3.14
-# loader = pkgutil.get_loader("not1mm")
-# WORKING_PATH = os.path.dirname(loader.get_filename())
-WORKING_PATH = os.path.dirname(__loader__.get_filename())
-
-if "XDG_DATA_HOME" in os.environ:
-    DATA_PATH = os.environ.get("XDG_DATA_HOME")
-else:
-    DATA_PATH = str(Path.home() / ".local" / "share")
-DATA_PATH += "/not1mm"
-
-if "XDG_CONFIG_HOME" in os.environ:
-    CONFIG_PATH = os.environ.get("XDG_CONFIG_HOME")
-else:
-    CONFIG_PATH = str(Path.home() / ".config")
-CONFIG_PATH += "/not1mm"
-
-DARK_STYLESHEET = ""
-
-PREF = {}
-if os.path.exists(CONFIG_PATH + "/not1mm.json"):
-    with open(CONFIG_PATH + "/not1mm.json", "rt", encoding="utf-8") as file_descriptor:
-        PREF = loads(file_descriptor.read())
-
 
 class Band:
     """the band"""
@@ -108,16 +84,20 @@ class Database:
         self.db.row_factory = self.row_factory
         self.cursor = self.db.cursor()
         sql_command = (
-            "create table spots("
-            "ts DATETIME NOT NULL, "
+            "create table spots ("
             "callsign VARCHAR(15) NOT NULL, "
+            "ts DATETIME NOT NULL, "
             "freq DOUBLE NOT NULL, "
-            "band VARCHAR(6), "
             "mode VARCHAR(6), "
             "spotter VARCHAR(15) NOT NULL, "
             "comment VARCHAR(45));"
         )
         self.cursor.execute(sql_command)
+
+        self.cursor.execute("CREATE INDEX spot_call_index ON spots (callsign);")
+        self.cursor.execute("CREATE INDEX spot_freq_index ON spots (freq);")
+        self.cursor.execute("CREATE INDEX spot_ts_index ON spots (ts);")
+
         self.db.commit()
 
     @staticmethod
@@ -160,7 +140,7 @@ class Database:
             logger.debug("%s", exception)
             return {}
 
-    def addspot(self, spot: dict) -> None:
+    def addspot(self, spot: dict, erase=True) -> None:
         """
         Add spot to database, replacing any previous spots with the same call.
 
@@ -170,60 +150,38 @@ class Database:
         A dict of the form: {'ts': datetime, 'callsign': str, 'freq': float,
         'band': str,'mode': str,'spotter': str, 'comment': str}
 
+        erase: bool
+        If True, delete any previous spots with the same callsign.
+        If False, do not delete any previous spots with the same callsign.
+        Default is True.
+
         Returns
         -------
         Nothing.
         """
         try:
-            delete_call = (
-                f"delete from spots where callsign = '{spot.get('callsign')}';"
+            if spot.get("comment", None) == "MARKED":
+                delete_call = "delete from spots where callsign = ?;"
+                self.cursor.execute(delete_call, (spot.get("callsign"),))
+                self.db.commit()
+            elif erase:
+                delete_call = (
+                    "delete from spots where callsign = ? AND comment != 'MARKED';"
+                )
+                self.cursor.execute(delete_call, (spot.get("callsign"),))
+                self.db.commit()
+
+            self.cursor.execute(
+                "INSERT INTO spots(callsign, ts, freq, mode, spotter, comment) VALUES(?, ?, ?, ?, ?, ?)",
+                (
+                    spot["callsign"],
+                    spot["ts"],
+                    spot["freq"],
+                    spot.get("mode", None),
+                    spot["spotter"],
+                    spot.get("comment", None),
+                ),
             )
-            self.cursor.execute(delete_call)
-            self.db.commit()
-
-            pre = "INSERT INTO spots("
-            values = []
-            columns = ""
-            placeholders = ""
-            for key in spot.keys():
-                columns += f"{key},"
-                values.append(spot[key])
-                placeholders += "?,"
-            post = f") VALUES({placeholders[:-1]});"
-
-            sql = f"{pre}{columns[:-1]}{post}"
-            self.cursor.execute(sql, tuple(values))
-            self.db.commit()
-        except sqlite3.IntegrityError:
-            ...
-
-    def addmark(self, spot: dict) -> None:
-        """
-        Add spot to database, replacing any previous spots with the same call.
-
-        Parameters
-        ----------
-        spot: Dict
-        A dict of the form: {'ts': datetime, 'callsign': str, 'freq': float,
-        'band': str,'mode': str,'spotter': str, 'comment': str}
-
-        Returns
-        -------
-        Nothing.
-        """
-        try:
-            pre = "INSERT INTO spots("
-            values = []
-            columns = ""
-            placeholders = ""
-            for key in spot.keys():
-                columns += f"{key},"
-                values.append(spot[key])
-                placeholders += "?,"
-            post = f") VALUES({placeholders[:-1]});"
-
-            sql = f"{pre}{columns[:-1]}{post}"
-            self.cursor.execute(sql, tuple(values))
             self.db.commit()
         except sqlite3.IntegrityError:
             ...
@@ -263,7 +221,8 @@ class Database:
         A list of dicts.
         """
         self.cursor.execute(
-            f"select * from spots where freq >= {start} and freq <= {end} order by freq ASC;"
+            "select * from spots where freq >= ? and freq <= ? order by freq ASC;",
+            (start, end),
         )
         return self.cursor.fetchall()
 
@@ -283,9 +242,9 @@ class Database:
         -------
         A dict of the spot.
         """
-
         self.cursor.execute(
-            f"select * from spots where freq > {current} and freq <= {limit} order by freq ASC;"
+            "select * from spots where freq > ? and freq <= ? order by freq ASC;",
+            (current, limit),
         )
         return self.cursor.fetchone()
 
@@ -308,8 +267,8 @@ class Database:
         """
 
         self.cursor.execute(
-            f"select * from spots where freq >= {start} "
-            f"and freq <= {end} and callsign like '%{dx}%';"
+            "select * from spots where freq >= ? and freq <= ? and callsign like ?;",
+            (start, end, f"%{dx}%"),
         )
         return self.cursor.fetchone()
 
@@ -329,7 +288,8 @@ class Database:
         A list of dicts.
         """
         self.cursor.execute(
-            f"select * from spots where freq < {current} and freq >= {limit} order by freq DESC;"
+            "select * from spots where freq < ? and freq >= ? order by freq DESC;",
+            (current, limit),
         )
         return self.cursor.fetchone()
 
@@ -347,14 +307,17 @@ class Database:
         None
         """
         self.cursor.execute(
-            f"delete from spots where ts < datetime('now', '-{minutes} minutes');"
+            "delete from spots where ts < datetime('now', ?);",
+            (f"-{minutes} minutes",),
         )
 
+    def delete_marks(self) -> None:
+        """Delete marked spots."""
+        self.cursor.execute("delete from spots where ts > datetime('now');")
 
-class MainWindow(QtWidgets.QMainWindow):
-    """
-    The main window
-    """
+
+class BandMapWindow(QDockWidget):
+    """The BandMapWindow class."""
 
     zoom = 5
     currentBand = Band("20m")
@@ -370,17 +333,23 @@ class MainWindow(QtWidgets.QMainWindow):
     bandwidth_mark = []
     worked_list = {}
     multicast_interface = None
+    text_color = QColor(45, 45, 45)
+    cluster_expire = pyqtSignal(str)
+    message = pyqtSignal(dict)
 
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
+    def __init__(self):
+        super().__init__()
         self._udpwatch = None
-        data_path = WORKING_PATH + "/data/bandmap.ui"
-        uic.loadUi(data_path, self)
-        if PREF.get("dark_mode"):
-            self.setStyleSheet(DARK_STYLESHEET)
+
+        uic.loadUi(fsutils.APP_DATA_PATH / "bandmap.ui", self)
+        self.settings = self.get_settings()
+        self.clear_spot_olderSpinBox.setValue(
+            int(self.settings.get("cluster_expire", 1))
+        )
         self.agetime = self.clear_spot_olderSpinBox.value()
         self.clear_spot_olderSpinBox.valueChanged.connect(self.spot_aging_changed)
         self.clearButton.clicked.connect(self.clear_spots)
+        self.clearmarkedButton.clicked.connect(self.clear_marked)
         self.zoominButton.clicked.connect(self.dec_zoom)
         self.zoomoutButton.clicked.connect(self.inc_zoom)
         self.connectButton.clicked.connect(self.connect)
@@ -399,148 +368,182 @@ class MainWindow(QtWidgets.QMainWindow):
         self.update_timer = QtCore.QTimer()
         self.update_timer.timeout.connect(self.update_station_timer)
         self.update_timer.start(UPDATE_INTERVAL)
+        self.setDarkMode(self.settings.get("darkmode", False))
         self.update()
-        self.multicast_interface = Multicast(
-            PREF.get("multicast_group", "239.1.1.1"),
-            PREF.get("multicast_port", 2239),
-            PREF.get("interface_ip", "0.0.0.0"),
-        )
-        self.multicast_interface.ready_read_connect(self.watch_udp)
         self.request_workedlist()
+        self.request_contest()
 
-    def quit_app(self):
-        """doc"""
-        app.quit()
+    def get_settings(self) -> dict:
+        """Get the settings."""
+        if os.path.exists(fsutils.CONFIG_FILE):
+            with open(fsutils.CONFIG_FILE, "rt", encoding="utf-8") as file_descriptor:
+                return loads(file_descriptor.read())
+
+    def msg_from_main(self, packet):
+        """"""
+        if packet.get("cmd", "") == "RADIO_STATE":
+            self.set_band(packet.get("band") + "m", False)
+            try:
+                if self.rx_freq != float(packet.get("vfoa")) / 1000000:
+                    self.rx_freq = float(packet.get("vfoa")) / 1000000
+                    self.tx_freq = self.rx_freq
+                    self.center_on_rxfreq()
+            except ValueError:
+                print(f"vfo value error {packet.get('vfoa')}")
+                logger.debug(f"vfo value error {packet.get('vfoa')}")
+                return
+            bw_returned = packet.get("bw", "0")
+            if not bw_returned.isdigit():
+                bw_returned = "0"
+            self.bandwidth = int(bw_returned)
+            step, _ = self.determine_step_digits()
+            self.drawTXRXMarks(step)
+            return
+        if packet.get("cmd", "") == "NEXTSPOT" and self.rx_freq:
+            spot = self.spots.get_next_spot(
+                self.rx_freq + 0.000001, self.currentBand.end
+            )
+            if spot:
+                cmd = {}
+                cmd["cmd"] = "TUNE"
+                cmd["freq"] = spot.get("freq", self.rx_freq)
+                cmd["spot"] = spot.get("callsign", "")
+                self.message.emit(cmd)
+            return
+        if packet.get("cmd", "") == "PREVSPOT" and self.rx_freq:
+            spot = self.spots.get_prev_spot(
+                self.rx_freq - 0.000001, self.currentBand.start
+            )
+            if spot:
+                cmd = {}
+                cmd["cmd"] = "TUNE"
+                cmd["freq"] = spot.get("freq", self.rx_freq)
+                cmd["spot"] = spot.get("callsign", "")
+                self.message.emit(cmd)
+            return
+        if packet.get("cmd", "") == "SPOTDX":
+            dx = packet.get("dx", "")
+            freq = packet.get("freq", 0.0)
+            spotdx = f"dx {dx} {freq}"
+            self.send_command(spotdx)
+            return
+        if packet.get("cmd", "") == "MARKDX":
+            dx = packet.get("dx", "")
+            freq = packet.get("freq", 0.0)
+            the_UTC_time = datetime.now(timezone.utc).isoformat(" ")[:19].split()[1]
+            spot = {
+                "ts": "2099-01-01 " + the_UTC_time,
+                "callsign": dx,
+                "freq": freq / 1000,
+                "band": self.currentBand.name,
+                "mode": "DX",
+                "spotter": platform.node(),
+                "comment": "MARKED",
+            }
+            self.spots.addspot(spot, erase=False)
+            self.update_stations()
+            return
+
+        if packet.get("cmd", "") == "FINDDX":
+            dx = packet.get("dx", "")
+            spot = self.spots.get_matching_spot(
+                dx, self.currentBand.start, self.currentBand.end
+            )
+            if spot:
+                cmd = {}
+                cmd["cmd"] = "TUNE"
+                cmd["freq"] = spot.get("freq", self.rx_freq)
+                cmd["spot"] = spot.get("callsign", "")
+                self.message.emit(cmd)
+            return
+        if packet.get("cmd", "") == "WORKED":
+            self.worked_list = packet.get("worked", {})
+            logger.debug("%s", f"{self.worked_list}")
+            return
+        if packet.get("cmd", "") == "CALLCHANGED":
+            call = packet.get("call", "")
+            if call:
+                result = self.spots.get_like_calls(call)
+                if result:
+                    cmd = {}
+                    cmd["cmd"] = "CHECKSPOTS"
+                    cmd["spots"] = result
+                    self.message.emit(cmd)
+                    return
+            cmd = {}
+            cmd["cmd"] = "CHECKSPOTS"
+            cmd["spots"] = []
+            self.message.emit(cmd)
+            return
+        if packet.get("cmd", "") == "CONTESTSTATUS":
+            if not self.callsignField.text():
+                self.callsignField.setText(packet.get("operator", "").upper())
+            return
+        if packet.get("cmd", "") == "DARKMODE":
+            self.setDarkMode(packet.get("state", False))
+
+    def setDarkMode(self, setdarkmode=False):
+        """Set dark mode"""
+        if setdarkmode:
+            darkPalette = QPalette()
+            darkColor = QColor(56, 56, 56)
+            self.text_color = QColorConstants.White
+            disabledColor = QColor(127, 127, 127)
+            darkPalette.setColor(QPalette.ColorRole.Window, darkColor)
+            darkPalette.setColor(QPalette.ColorRole.WindowText, QColorConstants.White)
+            darkPalette.setColor(QPalette.ColorRole.Base, QColor(45, 45, 45))
+            darkPalette.setColor(QPalette.ColorRole.AlternateBase, darkColor)
+            darkPalette.setColor(QPalette.ColorRole.Text, QColorConstants.White)
+            darkPalette.setColor(QPalette.ColorRole.Button, darkColor)
+            darkPalette.setColor(QPalette.ColorRole.ButtonText, QColorConstants.White)
+            darkPalette.setColor(QPalette.ColorRole.BrightText, QColorConstants.Red)
+            darkPalette.setColor(QPalette.ColorRole.Link, QColor(42, 130, 218))
+            darkPalette.setColor(QPalette.ColorRole.Highlight, QColor(42, 130, 218))
+            darkPalette.setColor(
+                QPalette.ColorRole.HighlightedText, QColorConstants.Black
+            )
+            darkPalette.setColor(
+                QPalette.ColorGroup.Disabled,
+                QPalette.ColorRole.ButtonText,
+                disabledColor,
+            )
+            darkPalette.setColor(
+                QPalette.ColorGroup.Disabled,
+                QPalette.ColorRole.HighlightedText,
+                disabledColor,
+            )
+            darkPalette.setColor(
+                QPalette.ColorGroup.Disabled,
+                QPalette.ColorRole.Text,
+                disabledColor,
+            )
+
+            self.setPalette(darkPalette)
+            self.callsignField.setPalette(darkPalette)
+            self.update()
+        else:
+            palette = self.style().standardPalette()
+            self.setPalette(palette)
+            self.callsignField.setPalette(palette)
+            self.text_color = QColorConstants.Black
+            self.update()
 
     def connect(self):
-        """doc"""
-        if self.connected is True:
-            self.socket.close()
-            self.connected = False
-            self.connectButton.setStyleSheet("color: red;")
-            self.connectButton.setText("Closed")
+        """Connect to the cluster."""
+        if not self.callsignField.text():
+            self.callsignField.setFocus()
             return
-        if os.path.exists(CONFIG_PATH + "/not1mm.json"):
-            with open(
-                CONFIG_PATH + "/not1mm.json", "rt", encoding="utf-8"
-            ) as _file_descriptor:
-                globals()["PREF"] = loads(_file_descriptor.read())
-        server = PREF.get("cluster_server", "dxc.nc7j.com")
-        port = PREF.get("cluster_port", 7373)
-        logger.debug("%s", f"{server} {port}")
+        if self.connected is True:
+            self.close_cluster()
+            return
+        self.settings = self.get_settings()
+        server = self.settings.get("cluster_server", "dxc.nc7j.com")
+        port = self.settings.get("cluster_port", 7373)
+        logger.info(f"connecting to dx cluster {server} {port}")
         self.socket.connectToHost(server, port)
-        self.connectButton.setStyleSheet("color: white;")
         self.connectButton.setText("Connecting")
         self.connected = True
 
-    def watch_udp(self):
-        """doc"""
-        while self.multicast_interface.server_udp.hasPendingDatagrams():
-            packet = self.multicast_interface.read_datagram_as_json()
-
-            if packet.get("station", "") != platform.node():
-                continue
-            if packet.get("cmd", "") == "RADIO_STATE":
-                self.set_band(packet.get("band") + "m", False)
-                try:
-                    if self.rx_freq != float(packet.get("vfoa")) / 1000000:
-                        self.rx_freq = float(packet.get("vfoa")) / 1000000
-                        self.tx_freq = self.rx_freq
-                        self.center_on_rxfreq()
-                except ValueError:
-                    logger.debug(f"vfo value error {packet.get('vfoa')}")
-                    continue
-                bw_returned = packet.get("bw", "0")
-                if not bw_returned.isdigit():
-                    bw_returned = "0"
-                self.bandwidth = int(bw_returned)
-                step, _ = self.determine_step_digits()
-                self.drawTXRXMarks(step)
-                continue
-
-            if packet.get("cmd", "") == "NEXTSPOT" and self.rx_freq:
-                spot = self.spots.get_next_spot(
-                    self.rx_freq + 0.000001, self.currentBand.end
-                )
-                if spot:
-                    cmd = {}
-                    cmd["cmd"] = "TUNE"
-                    cmd["station"] = platform.node()
-                    cmd["freq"] = spot.get("freq", self.rx_freq)
-                    cmd["spot"] = spot.get("callsign", "")
-                    self.multicast_interface.send_as_json(cmd)
-                continue
-
-            if packet.get("cmd", "") == "PREVSPOT" and self.rx_freq:
-                spot = self.spots.get_prev_spot(
-                    self.rx_freq - 0.000001, self.currentBand.start
-                )
-                if spot:
-                    cmd = {}
-                    cmd["cmd"] = "TUNE"
-                    cmd["station"] = platform.node()
-                    cmd["freq"] = spot.get("freq", self.rx_freq)
-                    cmd["spot"] = spot.get("callsign", "")
-                    self.multicast_interface.send_as_json(cmd)
-                continue
-            if packet.get("cmd", "") == "SPOTDX":
-                dx = packet.get("dx", "")
-                freq = packet.get("freq", 0.0)
-                spotdx = f"dx {dx} {freq}"
-                self.send_command(spotdx)
-                continue
-            if packet.get("cmd", "") == "MARKDX":
-                dx = packet.get("dx", "")
-                freq = packet.get("freq", 0.0)
-                spot = {
-                    "ts": "2099-01-01 01:00:00",
-                    "callsign": dx,
-                    "freq": freq / 1000,
-                    "band": self.currentBand.name,
-                    "mode": "DX",
-                    "spotter": platform.node(),
-                    "comment": "DX",
-                }
-                self.spots.addmark(spot)
-                self.update_stations()
-                continue
-            if packet.get("cmd", "") == "FINDDX":
-                dx = packet.get("dx", "")
-                spot = self.spots.get_matching_spot(
-                    dx, self.currentBand.start, self.currentBand.end
-                )
-                if spot:
-                    cmd = {}
-                    cmd["cmd"] = "TUNE"
-                    cmd["station"] = platform.node()
-                    cmd["freq"] = spot.get("freq", self.rx_freq)
-                    cmd["spot"] = spot.get("callsign", "")
-                    self.multicast_interface.send_as_json(cmd)
-                continue
-            if packet.get("cmd", "") == "WORKED":
-                self.worked_list = packet.get("worked", {})
-                logger.debug("%s", f"{self.worked_list}")
-                continue
-            if packet.get("cmd", "") == "CALLCHANGED":
-                call = packet.get("call", "")
-                if call:
-                    result = self.spots.get_like_calls(call)
-                    if result:
-                        cmd = {}
-                        cmd["cmd"] = "CHECKSPOTS"
-                        cmd["station"] = platform.node()
-                        cmd["spots"] = result
-                        self.multicast_interface.send_as_json(cmd)
-                        continue
-                cmd = {}
-                cmd["cmd"] = "CHECKSPOTS"
-                cmd["station"] = platform.node()
-                cmd["spots"] = []
-                self.multicast_interface.send_as_json(cmd)
-                continue
-            if packet.get("cmd", "") == "HALT":
-                self.quit_app()
-
     def spot_clicked(self):
         """dunno"""
         items = self.bandmap_scene.selectedItems()
@@ -548,17 +551,21 @@ class MainWindow(QtWidgets.QMainWindow):
             if item:
                 cmd = {}
                 cmd["cmd"] = "TUNE"
-                cmd["station"] = platform.node()
                 cmd["freq"] = items[0].property("freq")
                 cmd["spot"] = items[0].toPlainText().split()[0]
-                self.multicast_interface.send_as_json(cmd)
+                self.message.emit(cmd)
 
     def request_workedlist(self):
         """Request worked call list from logger"""
         cmd = {}
         cmd["cmd"] = "GETWORKEDLIST"
-        cmd["station"] = platform.node()
-        self.multicast_interface.send_as_json(cmd)
+        self.message.emit(cmd)
+
+    def request_contest(self):
+        """Request active contest from logger"""
+        cmd = {}
+        cmd["cmd"] = "GETCONTESTSTATUS"
+        self.message.emit(cmd)
 
     def update_station_timer(self):
         """doc"""
@@ -566,7 +573,10 @@ class MainWindow(QtWidgets.QMainWindow):
 
     def update(self):
         """doc"""
-        self.update_timer.setInterval(UPDATE_INTERVAL)
+        try:
+            self.update_timer.setInterval(UPDATE_INTERVAL)
+        except AttributeError:
+            ...
         self.clear_all_callsign_from_scene()
         self.clear_freq_mark(self.rxMark)
         self.clear_freq_mark(self.txMark)
@@ -586,12 +596,13 @@ class MainWindow(QtWidgets.QMainWindow):
                 i * PIXELSPERSTEP,
                 length + 10,
                 i * PIXELSPERSTEP,
-                QtGui.QPen(QtGui.QColor(192, 192, 192)),
+                QtGui.QPen(self.text_color),
             )
             if i % 5 == 0:  # Add Frequency
                 freq = self.currentBand.start + step * i
                 text = f"{freq:.3f}"
                 self.something = self.bandmap_scene.addText(text)
+                self.something.setDefaultTextColor(self.text_color)
                 self.something.setPos(
                     -(self.something.boundingRect().width()) + 10,
                     i * PIXELSPERSTEP - (self.something.boundingRect().height() / 2),
@@ -626,15 +637,13 @@ class MainWindow(QtWidgets.QMainWindow):
             self.clear_freq_mark(self.bandwidth_mark)
             self.clear_freq_mark(self.rxMark)
             self.draw_bandwidth(
-                self.rx_freq, step, QtGui.QColor(30, 30, 180, 180), self.bandwidth_mark
-            )
-            self.drawfreqmark(
-                self.rx_freq, step, QtGui.QColor(30, 180, 30, 180), self.rxMark
+                self.rx_freq, step, QColor(30, 30, 180, 180), self.bandwidth_mark
             )
+            self.drawfreqmark(self.rx_freq, step, QColor(30, 180, 30, 180), self.rxMark)
 
     def Freq2ScenePos(self, freq: float):
         """doc"""
-        if freq < self.currentBand.start or freq > self.currentBand.end:
+        if not freq or freq < self.currentBand.start or freq > self.currentBand.end:
             return QtCore.QPointF()
         step, _digits = self.determine_step_digits()
         ret = QtCore.QPointF(
@@ -650,22 +659,11 @@ class MainWindow(QtWidgets.QMainWindow):
     def center_on_rxfreq(self):
         """doc"""
         freq_pos = self.Freq2ScenePos(self.rx_freq).y()
-        # self.graphicsView.verticalScrollBar().setSliderPosition(
-        #     int(freq_pos - (self.height() / 2) + 80)
-        # )
         self.scrollArea.verticalScrollBar().setValue(
             int(freq_pos - (self.height() / 2) + 80)
         )
-        # This does not work... Have no idea why.
-        # anim = QtCore.QPropertyAnimation(
-        #     self.scrollArea.verticalScrollBar(), "value".encode()
-        # )
-        # anim.setDuration(300)
-        # anim.setStartValue(self.scrollArea.verticalScrollBar().value())
-        # anim.setEndValue(int(freq_pos - (self.height() / 2) + 80))
-        # anim.start(QtCore.QAbstractAnimation.DeletionPolicy.DeleteWhenStopped)
 
-    def drawfreqmark(self, freq, _step, color, currentPolygon):
+    def drawfreqmark(self, freq, _step, color, currentPolygon) -> None:
         """doc"""
 
         self.clear_freq_mark(currentPolygon)
@@ -684,14 +682,14 @@ class MainWindow(QtWidgets.QMainWindow):
         brush = QtGui.QBrush(color)
         currentPolygon.append(self.bandmap_scene.addPolygon(poly, pen, brush))
 
-    def draw_bandwidth(self, freq, _step, color, currentPolygon):
+    def draw_bandwidth(self, freq, _step, color, currentPolygon) -> None:
         """bandwidth"""
         logger.debug("%s", f"mark:{currentPolygon} f:{freq} b:{self.bandwidth}")
         self.clear_freq_mark(currentPolygon)
         if freq < self.currentBand.start or freq > self.currentBand.end:
             return
         if freq and self.bandwidth:
-            # color = QtGui.QColor(30, 30, 180)
+            # color = QColor(30, 30, 180)
             bw_start = Decimal(str(freq)) - (
                 (Decimal(str(self.bandwidth)) / 2) / 1000000
             )
@@ -716,15 +714,21 @@ class MainWindow(QtWidgets.QMainWindow):
         step, _digits = self.determine_step_digits()
 
         result = self.spots.getspotsinband(self.currentBand.start, self.currentBand.end)
+        logger.debug(
+            f"{len(result)} spots in range {self.currentBand.start} - {self.currentBand.end}"
+        )
+
         entity = ""
         if result:
             min_y = 0.0
             for items in result:
-                pen_color = QtGui.QColor(192, 192, 192)
+                pen_color = self.text_color
+                if items.get("comment") == "MARKED":
+                    pen_color = QColor(47, 47, 255)
                 if items.get("callsign") in self.worked_list:
                     call_bandlist = self.worked_list.get(items.get("callsign"))
                     if self.currentBand.altname in call_bandlist:
-                        pen_color = QtGui.QColor(255, 47, 47)
+                        pen_color = QColor(255, 47, 47)
                 freq_y = (
                     (items.get("freq") - self.currentBand.start) / step
                 ) * PIXELSPERSTEP
@@ -744,8 +748,8 @@ class MainWindow(QtWidgets.QMainWindow):
                 text.document().setDocumentMargin(0)
                 text.setPos(60, text_y - (text.boundingRect().height() / 2))
                 text.setFlags(
-                    QtWidgets.QGraphicsItem.ItemIsFocusable
-                    | QtWidgets.QGraphicsItem.ItemIsSelectable
+                    QtWidgets.QGraphicsItem.GraphicsItemFlag.ItemIsFocusable
+                    | QtWidgets.QGraphicsItem.GraphicsItemFlag.ItemIsSelectable
                     | text.flags()
                 )
                 text.setProperty("freq", items.get("freq"))
@@ -776,23 +780,22 @@ class MainWindow(QtWidgets.QMainWindow):
 
         return (step, digits)
 
-    def set_band(self, band: str, savePrevBandZoom: bool):
-        """doc"""
+    def set_band(self, band: str, savePrevBandZoom: bool) -> None:
+        """Change band being shown."""
         logger.debug("%s", f"{band} {savePrevBandZoom}")
         if band != self.currentBand.name:
             if savePrevBandZoom:
                 self.saveCurrentZoom()
             self.currentBand = Band(band)
-            # self.zoom = self.savedZoom(band)
             self.update()
 
-    def spot_aging(self):
-        """doc"""
+    def spot_aging(self) -> None:
+        """Delete spots older than age time."""
         if self.agetime:
             self.spots.delete_spots(self.agetime)
 
-    def clear_all_callsign_from_scene(self):
-        """doc"""
+    def clear_all_callsign_from_scene(self) -> None:
+        """Remove callsigns from the scene."""
         for items in self.textItemList:
             self.bandmap_scene.removeItem(items)
         self.textItemList.clear()
@@ -800,70 +803,70 @@ class MainWindow(QtWidgets.QMainWindow):
             self.bandmap_scene.removeItem(items)
         self.lineitemlist.clear()
 
-    def clear_freq_mark(self, currentPolygon):
-        """doc"""
+    def clear_freq_mark(self, currentPolygon) -> None:
+        """Remove frequency marks from the scene."""
         if currentPolygon:
             for mark in currentPolygon:
                 self.bandmap_scene.removeItem(mark)
         currentPolygon.clear()
 
-    def receive(self):
-        """doc"""
-        data = self.socket.readAll()
-        data = str(data, "utf-8").strip()
-        if "login:" in data:
-            self.send_command(self.callsignField.text())
-            self.send_command(PREF.get("cluster_filter", ""))
-            self.send_command("set dx extension Section")
-            self.send_command("set dx mode " + PREF.get("cluster_mode", "OPEN"))
-            return
-        if "call:" in data or "callsign:" in data:
-            self.send_command(self.callsignField.text())
-            self.send_command(PREF.get("cluster_filter", ""))
-            self.send_command("set dx extension Section")
-            self.send_command("set dx mode " + PREF.get("cluster_mode", "OPEN"))
-            return
-        if "DX de" in data:
-            parts = data.split()
-            spotter = parts[2]
-            freq = parts[3]
-            dx = parts[4]
-            _time = parts[-1]
-            comment = " ".join(parts[5:-1])
-            # spot = DxSpot()
-            spot = {}
-            spot["ts"] = datetime.now(timezone.utc).isoformat(" ")[:19]
-            spot["callsign"] = dx
-            spot["spotter"] = spotter
-            spot["comment"] = comment
+    def receive(self) -> None:
+        """Process waiting bytes"""
+        while self.socket.bytesAvailable():
+            data = self.socket.readLine(1000)
             try:
-                spot["freq"] = float(freq) / 1000
-            except ValueError:
-                logger.debug("%s", f"{data}")
-            self.spots.addspot(spot)
-            return
-        if self.callsignField.text().upper() in data:
-            self.connectButton.setStyleSheet("color: green;")
-            self.connectButton.setText("Connected")
-            logger.debug("%s", f"{data}")
+                data = str(data, "utf-8").strip()
+            except UnicodeDecodeError:
+                continue
 
-    def maybeconnected(self):
-        """doc"""
-        self.connectButton.setStyleSheet("color: yellow;")
+            if "login:" in data or "call:" in data or "callsign:" in data:
+                self.send_command(self.callsignField.text())
+                self.send_command(self.settings.get("cluster_filter", ""))
+                self.send_command("set dx extension Section")
+                self.send_command(
+                    "set dx mode " + self.settings.get("cluster_mode", "OPEN")
+                )
+                return
+            if "BEACON" in data:
+                return
+            if "DX de" in data:
+                parts = data.split()
+                spotter = parts[2]
+                freq = parts[3]
+                dx = parts[4]
+                _time = parts[-1]
+                comment = " ".join(parts[5:-1])
+                spot = {}
+                spot["ts"] = datetime.now(timezone.utc).isoformat(" ")[:19]
+                spot["callsign"] = dx
+                spot["spotter"] = spotter
+                spot["comment"] = comment
+                logger.debug(f"{spot}")
+                try:
+                    spot["freq"] = float(freq) / 1000
+                    self.spots.addspot(spot)
+                except ValueError:
+                    logger.debug(f"couldn't parse freq from datablock {data}")
+                return
+            if self.callsignField.text().upper() in data:
+                self.connectButton.setText("Connected")
+                logger.debug(f"callsign login acknowledged {data}")
+
+    def maybeconnected(self) -> None:
+        """Update visual state of the connect button."""
         self.connectButton.setText("Connecting")
 
-    def socket_error(self):
-        """doc"""
-        print("An Error occurred.")
+    def socket_error(self) -> None:
+        """Oopsie"""
+        logger.warning("An Error occurred.")
 
-    def disconnected(self):
-        """doc"""
+    def disconnected(self) -> None:
+        """Called when socket is disconnected."""
         self.connected = False
-        self.connectButton.setStyleSheet("color: red;")
         self.connectButton.setText("Closed")
 
-    def send_command(self, cmd: str):
-        """doc"""
+    def send_command(self, cmd: str) -> None:
+        """Send a command to the cluster."""
         cmd += "\r\n"
         tosend = bytes(cmd, encoding="ascii")
         logger.debug("%s", f"{tosend}")
@@ -871,44 +874,30 @@ class MainWindow(QtWidgets.QMainWindow):
             if self.socket.isOpen():
                 self.socket.write(tosend)
 
-    def clear_spots(self):
-        """doc"""
+    def clear_spots(self) -> None:
+        """Delete all spots from the database."""
         self.spots.delete_spots(0)
 
-    def spot_aging_changed(self):
-        """doc"""
+    def clear_marked(self) -> None:
+        """Delete all marked spots."""
+        self.spots.delete_marks()
+
+    def spot_aging_changed(self) -> None:
+        """Called when spot aging spinbox is changed."""
         self.agetime = self.clear_spot_olderSpinBox.value()
+        self.cluster_expire.emit(str(self.agetime))
 
-    def showContextMenu(self):
-        """doc"""
+    def showContextMenu(self) -> None:
+        """doc string for the linter"""
 
+    def close_cluster(self) -> None:
+        """Close socket connection"""
+        if self.socket and self.socket.isOpen():
+            logger.info("Closing dx cluster connection")
+            self.socket.close()
+            self.connected = False
+            self.connectButton.setText("Closed")
 
-def run():
-    """doc"""
-    sys.exit(app.exec())
-
-
-logger = logging.getLogger("__main__")
-handler = logging.StreamHandler()
-formatter = logging.Formatter(
-    datefmt="%H:%M:%S",
-    fmt="[%(asctime)s] %(levelname)s %(module)s - %(funcName)s Line %(lineno)d:\n%(message)s",
-)
-handler.setFormatter(formatter)
-logger.addHandler(handler)
-
-if Path("./debug").exists():
-    logger.setLevel(logging.DEBUG)
-    logger.debug("debugging on")
-else:
-    logger.setLevel(logging.WARNING)
-    logger.warning("debugging off")
-
-app = QtWidgets.QApplication(sys.argv)
-
-
-app.setStyle("Adwaita-Dark")
-window = MainWindow()
-window.show()
-if __name__ == "__main__":
-    run()
+    def closeEvent(self, _event: QtGui.QCloseEvent) -> None:
+        """Triggered when instance closes."""
+        self.close_cluster()
