@@ -1,50 +1,36 @@
 #!/usr/bin/env python3
 """
-Display current log
+not1mm Contest logger
+Email: michael.bridak@gmail.com
+GPL V3
+Class: LogWindow
+Purpose: Onscreen widget to show and edit logged contacts. 
 """
 # pylint: disable=no-name-in-module, unused-import, no-member, c-extension-no-member
 # pylint: disable=logging-fstring-interpolation, too-many-lines
 # QTableWidget
 # focusedLog, generalLog
+
 import logging
-import math
 import os
-
-# import pkgutil
-import platform
 import queue
-import sys
+from json import loads
 
-from json import JSONDecodeError, loads, dumps
-from pathlib import Path
-
-from PyQt5 import QtCore, QtGui, QtWidgets, uic, Qt
-from PyQt5.QtCore import QDir, QItemSelectionModel
-from PyQt5 import QtNetwork
-from PyQt5.QtGui import QFontDatabase
+import math
+from PyQt6 import QtCore, QtGui, QtWidgets, uic
+from PyQt6.QtCore import QItemSelectionModel
+from PyQt6.QtWidgets import QDockWidget
+from PyQt6.QtGui import QColorConstants, QPalette, QColor
+from PyQt6.QtCore import pyqtSignal
 
+import not1mm.fsutils as fsutils
 from not1mm.lib.database import DataBase
-from not1mm.lib.multicast import Multicast
 from not1mm.lib.edit_contact import EditContact
+
+# from not1mm.lib.multicast import Multicast
 from not1mm.lib.n1mm import N1MM
 
-os.environ["QT_QPA_PLATFORMTHEME"] = "gnome"
-
-WORKING_PATH = os.path.dirname(__loader__.get_filename())
-
-if "XDG_DATA_HOME" in os.environ:
-    DATA_PATH = os.environ.get("XDG_DATA_HOME")
-else:
-    DATA_PATH = str(Path.home() / ".local" / "share")
-DATA_PATH += "/not1mm"
-
-if "XDG_CONFIG_HOME" in os.environ:
-    CONFIG_PATH = os.environ.get("XDG_CONFIG_HOME")
-else:
-    CONFIG_PATH = str(Path.home() / ".config")
-CONFIG_PATH += "/not1mm"
-
-DARK_STYLESHEET = ""
+logger = logging.getLogger(__name__)
 
 
 def safe_float(the_input: any, default=0.0) -> float:
@@ -72,14 +58,16 @@ def safe_float(the_input: any, default=0.0) -> float:
     return default
 
 
-class MainWindow(QtWidgets.QMainWindow):
+class LogWindow(QDockWidget):
     """
     The main window
     """
 
+    message = pyqtSignal(dict)
     multicast_interface = None
     dbname = None
     edit_contact_dialog = None
+    current_palette = None
     pref = {}
     columns = {
         0: "YYYY-MM-DD HH:MM:SS",
@@ -106,31 +94,34 @@ class MainWindow(QtWidgets.QMainWindow):
         21: "UUID",
     }
 
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
+    def __init__(self):
+        super().__init__()
         self.table_loading = True
         self._udpwatch = None
         self.udp_fifo = queue.Queue()
         self.n1mm = None
         self.load_pref()
-        if self.pref.get("dark_mode"):
-            self.setStyleSheet(DARK_STYLESHEET)
-        self.dbname = DATA_PATH + "/" + self.pref.get("current_database", "ham.db")
-        self.database = DataBase(self.dbname, WORKING_PATH)
+
+        self.dbname = fsutils.USER_DATA_PATH / self.pref.get(
+            "current_database", "ham.db"
+        )
+        self.database = DataBase(self.dbname, fsutils.USER_DATA_PATH)
+
         self.database.current_contest = self.pref.get("contest", 0)
         self.contact = self.database.empty_contact
-        data_path = WORKING_PATH + "/data/logwindow.ui"
-        uic.loadUi(data_path, self)
+        uic.loadUi(fsutils.APP_DATA_PATH / "logwindow.ui", self)
         self.setWindowTitle(
-            f"Log Display - {self.pref.get('current_database', 'ham.db')}"
+            f"QSO History - {self.pref.get('current_database', 'ham.db')}"
         )
         self.generalLog.setColumnCount(len(self.columns))
         self.focusedLog.setColumnCount(len(self.columns))
-        icon_path = WORKING_PATH + "/data/"
-        self.checkmark = QtGui.QPixmap(icon_path + "check.png")
+
+        self.checkmark = QtGui.QPixmap(str(fsutils.APP_DATA_PATH / "check.png"))
         self.checkicon = QtGui.QIcon()
         self.checkicon.addPixmap(self.checkmark)
-        self.generalLog.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
+        self.generalLog.setContextMenuPolicy(
+            QtCore.Qt.ContextMenuPolicy.CustomContextMenu
+        )
         self.generalLog.customContextMenuRequested.connect(self.edit_contact_selected)
         for column_number, column_name in self.columns.items():
             self.generalLog.setHorizontalHeaderItem(
@@ -142,8 +133,12 @@ class MainWindow(QtWidgets.QMainWindow):
 
         self.generalLog.cellDoubleClicked.connect(self.double_clicked)
         self.generalLog.cellChanged.connect(self.cell_changed)
-
-        self.focusedLog.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
+        self.generalLog.horizontalHeader().sectionResized.connect(
+            self.resize_headers_to_match
+        )
+        self.focusedLog.setContextMenuPolicy(
+            QtCore.Qt.ContextMenuPolicy.CustomContextMenu
+        )
         self.focusedLog.customContextMenuRequested.connect(
             self.edit_focused_contact_selected
         )
@@ -152,6 +147,7 @@ class MainWindow(QtWidgets.QMainWindow):
 
         for log in (self.generalLog, self.focusedLog):
             log.setColumnWidth(self.get_column("YYYY-MM-DD HH:MM:SS"), 200)
+
             log.setColumnWidth(self.get_column("Snt"), 50)
             log.setColumnWidth(self.get_column("Rcv"), 50)
             log.setColumnWidth(self.get_column("SentNr"), 75)
@@ -168,24 +164,80 @@ class MainWindow(QtWidgets.QMainWindow):
             log.verticalHeader().setVisible(False)
 
         self.get_log()
-        self.multicast_interface = Multicast(
-            self.pref.get("multicast_group", "239.1.1.1"),
-            self.pref.get("multicast_port", 2239),
-            self.pref.get("interface_ip", "0.0.0.0"),
-        )
-        self.multicast_interface.ready_read_connect(self.watch_udp)
-
         cmd = {}
         cmd["cmd"] = "GETCOLUMNS"
-        cmd["station"] = platform.node()
+        self.message.emit(cmd)
 
-        self.multicast_interface.send_as_json(cmd)
+    def msg_from_main(self, msg):
+        """"""
+        if msg.get("cmd", "") == "UPDATELOG":
+            logger.debug("External refresh command.")
+            self.get_log()
+        if msg.get("cmd", "") == "CALLCHANGED":
+            call = msg.get("call", "")
+            self.show_like_calls(call)
+        if msg.get("cmd", "") == "NEWDB":
+            self.load_new_db()
+        if msg.get("cmd", "") == "SHOWCOLUMNS":
+            for column in range(len(self.columns)):
+                self.generalLog.setColumnHidden(column, True)
+                self.focusedLog.setColumnHidden(column, True)
+            columns_to_show = msg.get("COLUMNS", [])
+            for column in columns_to_show:
+                if column == "Freq":
+                    column = "Freq (Khz)"
+                self.generalLog.setColumnHidden(self.get_column(column), False)
+                self.focusedLog.setColumnHidden(self.get_column(column), False)
+        if msg.get("cmd", "") == "DARKMODE":
+            self.set_dark_mode(msg.get("state", False))
 
-    def quit_app(self):
-        """
-        Quit the application.
-        """
-        app.quit()
+    def resize_headers_to_match(self) -> None:
+        """"""
+        for i in range(self.generalLog.columnCount()):
+            self.focusedLog.setColumnWidth(i, self.generalLog.columnWidth(i))
+
+    def set_dark_mode(self, dark: bool) -> None:
+        """Forces a darkmode palette."""
+
+        if dark:
+            darkPalette = QPalette()
+            darkColor = QColor(56, 56, 56)
+            disabledColor = QColor(127, 127, 127)
+            darkPalette.setColor(QPalette.ColorRole.Window, darkColor)
+            darkPalette.setColor(QPalette.ColorRole.WindowText, QColorConstants.White)
+            darkPalette.setColor(QPalette.ColorRole.Base, QColor(45, 45, 45))
+            darkPalette.setColor(QPalette.ColorRole.AlternateBase, darkColor)
+            darkPalette.setColor(QPalette.ColorRole.Text, QColorConstants.White)
+            darkPalette.setColor(QPalette.ColorRole.Button, darkColor)
+            darkPalette.setColor(QPalette.ColorRole.ButtonText, QColorConstants.White)
+            darkPalette.setColor(QPalette.ColorRole.BrightText, QColorConstants.Red)
+            darkPalette.setColor(QPalette.ColorRole.Link, QColor(42, 130, 218))
+            darkPalette.setColor(QPalette.ColorRole.Highlight, QColor(42, 130, 218))
+            darkPalette.setColor(
+                QPalette.ColorRole.HighlightedText, QColorConstants.Black
+            )
+            darkPalette.setColor(
+                QPalette.ColorGroup.Disabled,
+                QPalette.ColorRole.ButtonText,
+                disabledColor,
+            )
+            darkPalette.setColor(
+                QPalette.ColorGroup.Disabled,
+                QPalette.ColorRole.HighlightedText,
+                disabledColor,
+            )
+            darkPalette.setColor(
+                QPalette.ColorGroup.Disabled,
+                QPalette.ColorRole.Text,
+                disabledColor,
+            )
+
+            self.setPalette(darkPalette)
+            self.current_palette = darkPalette
+        else:
+            palette = self.style().standardPalette()
+            self.setPalette(palette)
+            self.current_palette = palette
 
     def get_column(self, name: str) -> int:
         """
@@ -218,9 +270,9 @@ class MainWindow(QtWidgets.QMainWindow):
         None
         """
         try:
-            if os.path.exists(CONFIG_PATH + "/not1mm.json"):
+            if os.path.exists(fsutils.CONFIG_FILE):
                 with open(
-                    CONFIG_PATH + "/not1mm.json", "rt", encoding="utf-8"
+                    fsutils.CONFIG_FILE, "rt", encoding="utf-8"
                 ) as file_descriptor:
                     self.pref = loads(file_descriptor.read())
                     logger.info("%s", self.pref)
@@ -244,6 +296,7 @@ class MainWindow(QtWidgets.QMainWindow):
         self.n1mm.send_lookup_packets = self.pref.get("send_n1mm_lookup", False)
         self.n1mm.send_score_packets = self.pref.get("send_n1mm_score", False)
         self.n1mm.radio_info["StationName"] = self.pref.get("n1mm_station_name", "")
+        self.set_dark_mode(self.pref.get("darkmode", False))
 
     def load_new_db(self) -> None:
         """
@@ -258,8 +311,10 @@ class MainWindow(QtWidgets.QMainWindow):
         None
         """
         self.load_pref()
-        self.dbname = DATA_PATH + "/" + self.pref.get("current_database", "ham.db")
-        self.database = DataBase(self.dbname, WORKING_PATH)
+        self.dbname = fsutils.USER_DATA_PATH / self.pref.get(
+            "current_database", "ham.db"
+        )
+        self.database = DataBase(self.dbname, fsutils.APP_DATA_PATH)
         self.database.current_contest = self.pref.get("contest", 0)
         self.contact = self.database.empty_contact
         self.get_log()
@@ -330,11 +385,17 @@ class MainWindow(QtWidgets.QMainWindow):
             "Sect": self.generalLog.item(row, self.get_column("Sect")).text(),
             "WPXPrefix": self.generalLog.item(row, self.get_column("WPX")).text(),
             "Power": self.generalLog.item(row, self.get_column("Power")).text(),
-            "IsMultiplier1": self.generalLog.item(row, self.get_column("M1")).text(),
+            "IsMultiplier1": int(
+                not bool(
+                    self.generalLog.item(row, self.get_column("M1")).icon().isNull()
+                )
+            ),
             "ZN": self.generalLog.item(row, self.get_column("ZN")).text(),
-            "IsMultiplier2": self.generalLog.item(row, self.get_column("M2"))
-            .text()
-            .upper(),
+            "IsMultiplier2": int(
+                not bool(
+                    self.generalLog.item(row, self.get_column("M2")).icon().isNull()
+                )
+            ),
             "CountryPrefix": self.generalLog.item(row, self.get_column("PFX")).text(),
             "Points": self.generalLog.item(row, self.get_column("PTS")).text(),
             "Name": self.generalLog.item(row, self.get_column("Name")).text(),
@@ -437,11 +498,17 @@ class MainWindow(QtWidgets.QMainWindow):
             "Sect": self.focusedLog.item(row, self.get_column("Sect")).text(),
             "WPXPrefix": self.focusedLog.item(row, self.get_column("WPX")).text(),
             "Power": self.focusedLog.item(row, self.get_column("Power")).text(),
-            "IsMultiplier1": self.focusedLog.item(row, self.get_column("M1")).text(),
+            "IsMultiplier1": int(
+                not bool(
+                    self.focusedLog.item(row, self.get_column("M1")).icon().isNull()
+                )
+            ),
             "ZN": self.focusedLog.item(row, self.get_column("ZN")).text(),
-            "IsMultiplier2": self.focusedLog.item(row, self.get_column("M2"))
-            .text()
-            .upper(),
+            "IsMultiplier2": int(
+                not bool(
+                    self.focusedLog.item(row, self.get_column("M2")).icon().isNull()
+                )
+            ),
             "CountryPrefix": self.focusedLog.item(row, self.get_column("PFX")).text(),
             "Points": self.focusedLog.item(row, self.get_column("PTS")).text(),
             "Name": self.focusedLog.item(row, self.get_column("Name")).text(),
@@ -555,9 +622,9 @@ class MainWindow(QtWidgets.QMainWindow):
         None
         """
         logger.debug("Edit: %s", uuid)
-        self.edit_contact_dialog = EditContact(WORKING_PATH)
-        if self.pref.get("dark_mode"):
-            self.edit_contact_dialog.setStyleSheet(DARK_STYLESHEET)
+        self.edit_contact_dialog = EditContact(fsutils.APP_DATA_PATH)
+        if self.current_palette:
+            self.edit_contact_dialog.setPalette(self.current_palette)
         self.edit_contact_dialog.accepted.connect(self.save_edited_contact)
         self.contact = self.database.fetch_contact_by_uuid(uuid)
         self.edit_contact_dialog.delete_2.clicked.connect(self.delete_contact)
@@ -735,7 +802,9 @@ class MainWindow(QtWidgets.QMainWindow):
             self.generalLog.setItem(
                 number_of_rows, self.get_column("YYYY-MM-DD HH:MM:SS"), first_item
             )
-            self.generalLog.setCurrentItem(first_item, QItemSelectionModel.NoUpdate)
+            self.generalLog.setCurrentItem(
+                first_item, QItemSelectionModel.SelectionFlag.NoUpdate
+            )
             self.generalLog.item(
                 number_of_rows, self.get_column("YYYY-MM-DD HH:MM:SS")
             ).setTextAlignment(0x0004 | 0x0080)
@@ -859,37 +928,6 @@ class MainWindow(QtWidgets.QMainWindow):
         self.generalLog.blockSignals(False)
         self.focusedLog.blockSignals(False)
 
-    def watch_udp(self) -> None:
-        """
-        Watch for UDP datagrams.
-        Parse commands from our platform.node().
-        """
-        while self.multicast_interface.server_udp.hasPendingDatagrams():
-            json_data = self.multicast_interface.read_datagram_as_json()
-
-            if json_data.get("station", "") != platform.node():
-                continue
-            if json_data.get("cmd", "") == "UPDATELOG":
-                logger.debug("External refresh command.")
-                self.get_log()
-            if json_data.get("cmd", "") == "CALLCHANGED":
-                call = json_data.get("call", "")
-                self.show_like_calls(call)
-            if json_data.get("cmd", "") == "NEWDB":
-                self.load_new_db()
-            if json_data.get("cmd", "") == "SHOWCOLUMNS":
-                for column in range(len(self.columns)):
-                    self.generalLog.setColumnHidden(column, True)
-                    self.focusedLog.setColumnHidden(column, True)
-                columns_to_show = json_data.get("COLUMNS", [])
-                for column in columns_to_show:
-                    if column == "Freq":
-                        column = "Freq (Khz)"
-                    self.generalLog.setColumnHidden(self.get_column(column), False)
-                    self.focusedLog.setColumnHidden(self.get_column(column), False)
-            if json_data.get("cmd", "") == "HALT":
-                self.quit_app()
-
     def show_like_calls(self, call: str) -> None:
         """
         Show all log entries that match call.
@@ -903,8 +941,10 @@ class MainWindow(QtWidgets.QMainWindow):
         -------
         None.
         """
+        self.focusedLog.blockSignals(True)
         if call == "":
             self.focusedLog.setRowCount(0)
+            self.focusedLog.blockSignals(False)
             return
         lines = self.database.fetch_like_calls(call)
         debug_line = f"{lines}"
@@ -918,7 +958,9 @@ class MainWindow(QtWidgets.QMainWindow):
             self.focusedLog.setItem(
                 number_of_rows, self.get_column("YYYY-MM-DD HH:MM:SS"), first_item
             )
-            self.focusedLog.setCurrentItem(first_item, QItemSelectionModel.NoUpdate)
+            self.focusedLog.setCurrentItem(
+                first_item, QItemSelectionModel.SelectionFlag.NoUpdate
+            )
             try:
                 self.focusedLog.item(
                     number_of_rows, self.get_column("YYYY-MM-DD HH:MM:SS")
@@ -1042,6 +1084,7 @@ class MainWindow(QtWidgets.QMainWindow):
                 self.get_column("UUID"),
                 QtWidgets.QTableWidgetItem(str(log_item.get("ID", ""))),
             )
+        self.focusedLog.blockSignals(False)
 
     def show_message_box(self, message: str) -> None:
         """
@@ -1061,56 +1104,3 @@ class MainWindow(QtWidgets.QMainWindow):
         message_box.setWindowTitle("Information")
         message_box.setStandardButtons(QtWidgets.QMessageBox.Ok)
         _ = message_box.exec_()
-
-
-def load_fonts_from_dir(directory: str) -> set:
-    """
-    Loads all fonts from a directory.
-
-    Parameters
-    ----------
-    directory : str
-    The directory to load fonts from.
-
-    Returns
-    -------
-    set
-    The set of font families.
-    """
-    font_families = set()
-    for _fi in QDir(directory).entryInfoList(["*.ttf", "*.woff", "*.woff2"]):
-        _id = QFontDatabase.addApplicationFont(_fi.absoluteFilePath())
-        font_families |= set(QFontDatabase.applicationFontFamilies(_id))
-    return font_families
-
-
-def main():
-    """main entry"""
-    sys.exit(app.exec())
-
-
-logger = logging.getLogger("__main__")
-handler = logging.StreamHandler()
-formatter = logging.Formatter(
-    datefmt="%H:%M:%S",
-    fmt="[%(asctime)s] %(levelname)s %(module)s - %(funcName)s Line %(lineno)d:\n%(message)s",
-)
-handler.setFormatter(formatter)
-logger.addHandler(handler)
-
-if Path("./debug").exists():
-    logger.setLevel(logging.DEBUG)
-    logger.debug("debugging on")
-else:
-    logger.setLevel(logging.WARNING)
-    logger.warning("debugging off")
-
-app = QtWidgets.QApplication(sys.argv)
-app.setStyle("Adwaita-Dark")
-font_path = WORKING_PATH + "/data"
-_families = load_fonts_from_dir(os.fspath(font_path))
-window = MainWindow()
-window.show()
-
-if __name__ == "__main__":
-    main()
